//
//  BFRecordScreenVideoController.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2021/12/14.
//

import Foundation
import GPUImage
import AVFoundation


class BFRecordScreenVideoManager : BFRecordScreenBaseManager{
    var movie: GPUImageMovie? // 视频预览
    var movieIsProcessing : Bool = false
    
    override func resetEnv() {
        guard let playerItem = recordItem?.playItem else {
            return
        }
        guard let preView = playView else {
            return
        }
        if movie != nil {
            cleanMovieTarget()
        }
        playerItem.seek(to: .zero, toleranceBefore: CMTime(seconds: 1, preferredTimescale: 1000000), toleranceAfter: CMTime(seconds: 1, preferredTimescale: 1000000), completionHandler: nil)
        movie = GPUImageMovie(playerItem: playerItem)
        //        movie?.runBenchmark = true
        movie?.playAtActualSpeed = true

        let filter = GPUImageFilter()
        movie?.addTarget(filter)
        filter.addTarget(preView)

        movie?.startProcessing()
        movieIsProcessing = true
        
        // 重置指针
        currentAssetProgress = .zero
        
        dele?.setAudioPlay(item: playerItem)
        
        super.resetEnv()

    }
    
    override func startRecord() {
        
        if !movieIsProcessing {
            movie?.startProcessing()
            movieIsProcessing = true
        }
        assetPlayer?.volume = 0
        assetPlayer?.play()

        super.startRecord()
    }
    
    override func pause() {
        super.pause()
    }
      
    override func play() {
        super.play()
        if !movieIsProcessing {
            movie?.startProcessing()
            movieIsProcessing = true
        }
        assetPlayer?.volume = 0
        assetPlayer?.play()
    }
    
    
    func cleanMovieTarget() {
        movie?.cancelProcessing()
        movieIsProcessing = false
        movie?.targets().forEach { target in
            if let objc = target as? GPUImageOutput {
                objc.removeAllTargets()
            }
        }
        movie?.removeAllTargets()
        movie?.removeFramebuffer()
    }
    
    override func clean() {
        cleanMovieTarget()
        super.clean()
    }
}
